#!/usr/bin/perl
# ----------------------------------------------------------------------
# | Module:     atom.pl                                                |
# |                                                                    |
# | Purpose:    Test wine Atom API calls                               |
# |                                                                    |
# |                                                                    |
# |                                                                    |
# ----------------------------------------------------------------------


# --------------------------------------------------------------
# | The stuff necessary to use the Wine/Perl interface         |
# --------------------------------------------------------------
use lib "wine";
use lib "wine/blib/arch";
use lib "wine/blib/lib";
use ExtUtils::testlib;
use wine;
use wine_util;



    # ----------------------------------------------------------------------
    # | Entry point (if lbpi is used)                                      |
    # |                                                                    |
    # |                                                                    |
    # |                                                                    |
    # ----------------------------------------------------------------------
    sub main
    {
        $wine = wine->new ("wine_callback");
        $wine->init;
    }



    # ----------------------------------------------------------------------
    # | Subroutine:  wine_callback                                         |
    # |                                                                    |
    # | Purpose:     Make various atom-related wine API calls              |
    # |                                                                    |
    # | Usage:       wine_callback                                         |
    # |                                                                    |
    # | Returns:     (none)                                                |
    # ----------------------------------------------------------------------
    sub wine_callback
    {
        # Locals
        my  ($atom, $w_atom);
        my  ($buf, $len);
        my  $i;
    
    # Begin sub wine_callback
        
        # --------------------------------------------------------------
        # | Initializations                                            |
        # --------------------------------------------------------------
        $wine->debug (0);
        $wine->module   ("kernel32");
        $wine->ret_type ("int");
        $name = "foobar";
        


        # --------------------------------------------------------------
        # | [GlobalAddAtom/GlobalFindAtom]                             |
        # |                                                            |
        # |                                                            |
        # --------------------------------------------------------------

            # --------------------------------------------------------------
            # | Add an atom                                                |
            # --------------------------------------------------------------
            $atom = $wine->call 
            (
                function => "GlobalAddAtomA",
                args     => $name
            );
            assert (($atom >= 0xc000) && ($atom <= 0xffff));

            # --------------------------------------------------------------
            # | Verify that it can be found (or not) appropriately         |
            # --------------------------------------------------------------
            $wine->function ("GlobalFindAtomA");
            assert ($wine->call (args => $name) == $atom);
            assert ($wine->call (args => uc ($name)) == $atom);
            assert (! $wine->call (args => "_$name"));

            # --------------------------------------------------------------
            # | Add the same atom, specifying string as unicode; should    |
            # | find the first one, not add a new one                      |
            # --------------------------------------------------------------
            $w_atom = $wine->call 
            (
                function => "GlobalAddAtomW",
                args     => wc ($name)
            );
            assert ($w_atom == $atom);

            # --------------------------------------------------------------
            # | Verify that it can be found (or not) appropriately via     |
            # | unicode name                                               |
            # --------------------------------------------------------------
            $wine->function ("GlobalFindAtomW");
            assert ($wine->call (args => wc ($name)) == $atom);
            assert ($wine->call (args => wc (uc ($name))) == $atom);
            assert (! $wine->call (args => wc ("_$name")));
        
            # --------------------------------------------------------------
            # | Test integer atoms                                         |
            # |                                                            |
            # | 0x0000 should be invalid; (0x0001 .. 0xbfff) should be     |
            # | valid; (0xc000 .. 0xffff) should be invalid                |
            # --------------------------------------------------------------
            $wine->module   ("kernel32");
            $wine->ret_type ("int");
#           for ($i = 0x0001; ($i <= 0xbfff); $i++)
#           {
#               assert ($wine->call (function => "GlobalAddAtomA", args => $i));
#               assert ($wine->call (function => "GlobalAddAtomW", args => $i));
#           }
#           for ($i = 0xc000; ($i <= 0xffff); $i++)
#           {
#               assert
#               (
#                   (! $wine->call (function => "GlobalAddAtomA", args => $i)) &&
#                   ($wine::err == 87)
#               );
#               assert
#               (
#                   (! $wine->call (function => "GlobalAddAtomA", args => $i)) &&
#                   ($wine::err == 87)
#               );
#           }
        


        # --------------------------------------------------------------
        # | [GlobalGetAtomName]                                        |
        # |                                                            |
        # |                                                            |
        # --------------------------------------------------------------

            # --------------------------------------------------------------
            # | Get the name of the atom we added above                    |
            # |                                                            |
            # | The second try specifies a buffer size that is too small   |
            # | to hold the atom name; I believe the API is supposed to    |
            # | copy as much of the name to the buffer as possible, and    |
            # | return how much it copied.  But empirically, it seems to   |
            # | return 0 instead ...                                       |
            # --------------------------------------------------------------
            $wine->function ("GlobalGetAtomNameA");
            $buf = "." x 20;
            $len = $wine->call (args => [ $atom, \$buf, length ($buf) ]);
            assert ($len == length ($name));
            assert
            (
                $buf eq
                ($name . "\x00" . ("." x (length ($buf) - length ($name) - 1)))
            );
            $buf = "." x 20;
            assert (! $wine->call (args => [ $atom, \$buf, 3 ]));

            # --------------------------------------------------------------
            # | Repeat, unicode-style                                      |
            # --------------------------------------------------------------
            $wine->function ("GlobalGetAtomNameW");
            $buf = "." x 20;
            $len = $wine->call (args => [ $atom, \$buf, (length ($buf) / 2) ]);
            assert ($len == wclen (wc ($name)));
            assert
            (
                $buf eq
                (wc ($name) . ("." x wclen (wc ($name))))
            );
            $buf = "." x 20;
            assert (! $wine->call (args => [ $atom, \$buf, 3 ]));

            # --------------------------------------------------------------
            # | Check error code returns                                   |
            # --------------------------------------------------------------
            $wine->function ("GlobalGetAtomNameA");
            assert (! ($wine->call (args => [ $atom, $buf,  0 ])) && ($wine::err == 234));
            assert (! ($wine->call (args => [ $atom, $buf, -1 ])) && ($wine::err == 234));
            $wine->function ("GlobalGetAtomNameW");
            assert (! ($wine->call (args => [ $atom, $buf,  0 ])) && ($wine::err == 234));
            assert (! ($wine->call (args => [ $atom, $buf, -1 ])) && ($wine::err == 234));
        
            # --------------------------------------------------------------
            # | Test integer atoms                                         |
            # --------------------------------------------------------------
            $wine->module   ("kernel32");
            $wine->function ("GlobalGetAtomNameA");
            $wine->ret_type ("int");
            $buf = "a" x 10;
#           for ($i = 1; ($i <= 0xbfff); $i++)
#           {
#               $len = $wine->call ( args => [ $i, \$buf, length ($buf) ] );
#               assert (($len > 1) && ($len < 7));
#               assert
#               (
#                   ($buf eq ("#$i\x00" . "a" x (10 - $len - 1)))
#               );
#           }
        


        # --------------------------------------------------------------
        # | [GlobalDeleteAtom]                                         |
        # |                                                            |
        # |                                                            |
        # --------------------------------------------------------------
        $name = "barkbar";
        $wine->module   ("kernel32");
        $wine->function ("GlobalDeleteAtom");
        $wine->ret_type ("int");
        $n = 5;

            # --------------------------------------------------------------
            # | Add some atoms (a molecule, if you will ...)               |
            # --------------------------------------------------------------
            for (1 .. $n)
            {
                $atom = $wine->call
                (
                    function => "GlobalAddAtomA",
                    args     => $name
                );
            }

            # --------------------------------------------------------------
            # | Verify that they can be found                              |
            # --------------------------------------------------------------
            assert ($wine->call (function => "GlobalFindAtomA", args => $name));
            assert ($wine->call (function => "GlobalFindAtomW", args => wc ($name)));

            # --------------------------------------------------------------
            # | That many deletions should return zero                     |
            # --------------------------------------------------------------
            for (1 .. $n)
            {
                assert
                (
                    ! $wine->call (args => $atom)
                );
            }

            # --------------------------------------------------------------
            # | Another deletion, after they're all gone, should return    |
            # | non-zero (specifically, atom number), and the error const  |
            # | should be 6 (ERROR_INVALID_HANDLE)                         |
            # --------------------------------------------------------------
            assert
            (
                ($wine->call (args => $atom) == $atom) &&
                ($wine::err == 6)
            );

            # --------------------------------------------------------------
            # | Verify that they are not found anymore                     |
            # --------------------------------------------------------------
            assert (! $wine->call (function => "GlobalFindAtomA", args => $name));
            assert (! $wine->call (function => "GlobalFindAtomW", args => wc ($name)));
        


        # --------------------------------------------------------------
        # | Test error handling                                        |
        # |                                                            |
        # |                                                            |
        # --------------------------------------------------------------

            # --------------------------------------------------------------
            # | ASCII                                                      |
            # --------------------------------------------------------------
            $wine->module   ("kernel32");
            $wine->ret_type ("int");
            assert
            (
                $atom = $wine->call
                (
                    function => "GlobalAddAtomA",
                    args => ("a" x 255)
                )
            );
            assert (! $wine->call (function => "GlobalDeleteAtom", args => $atom));
            assert
            (
                (! $wine->call (function => "GlobalAddAtomA", args => ("a" x 256))) &&
                ($wine::err == 87)
            );
            assert
            (
                (! $wine->call (function => "GlobalFindAtomA", args => ("b" x 256))) &&
                ($wine::err == 87)
            );

            # --------------------------------------------------------------
            # | Unicode                                                    |
            # --------------------------------------------------------------
            $wine->module   ("kernel32");
            $wine->ret_type ("int");
            assert
            (
                $atom = $wine->call
                (
                    function => "GlobalAddAtomW",
                    args => wc ("B" x 255)
                )
            );
            assert (! $wine->call (function => "GlobalDeleteAtom", args => $atom));
            assert
            (
                (! $wine->call (function => "GlobalAddAtomW", args => wc ("B" x 256))) &&
                ($wine::err == 87)
            );
            assert
            (
                (! $wine->call (function => "GlobalFindAtomW", args => wc ("C" x 256))) &&
                ($wine::err == 87)
            );

            

    } # End sub wine_callback



    # ----------------------------------------------------------------------
    # | Allow for script to be called on its own                           |
    # |                                                                    |
    # |                                                                    |
    # |                                                                    |
    # ----------------------------------------------------------------------
    main ();
