#!/usr/bin/perl
# ----------------------------------------------------------------------
# | Module:     wti.pl                                                 |
# |                                                                    |
# | Purpose:    Interactive front-end for testing wine API calls       |
# |             through wine/perl gateway                              |
# |                                                                    |
# |                                                                    |
# |                                                                    |
# ----------------------------------------------------------------------


# --------------------------------------------------------------
# | The stuff necessary to use the Wine/Perl interface         |
# --------------------------------------------------------------
use lib "wine/blib/arch";
use lib "wine/blib/lib";
use ExtUtils::testlib;
use wine;
use wine_util;



    # ----------------------------------------------------------------------
    # | Entry point (if lbpi is used)                                      |
    # |                                                                    |
    # |                                                                    |
    # |                                                                    |
    # ----------------------------------------------------------------------
    sub main
    {
        $wine = wine->new ("wine_callback");

        $wine->module   ("kernel32");
        $wine->function ("GlobalAddAtomA");
        $wine->args     ("bark");
        $wine->ret_type ("int");
        $wine->debug    (1);

        $wine->init;
    }



    # ----------------------------------------------------------------------
    # | Subroutine:  wine_callback                                         |
    # |                                                                    |
    # | Purpose:     Effectively the mainline; the above call to           |
    # |              $wine->init() transfers control to the wine init      |
    # |              library, which calls back to here                     |
    # |                                                                    |
    # | Usage:       wine_callback                                         |
    # |                                                                    |
    # | Returns:     (none)                                                |
    # ----------------------------------------------------------------------
    sub wine_callback
    {
        # Locals
    
    # Begin sub wine_callback

        # --------------------------------------------------------------
        # | See if we have ReadLine available to us                    |
        # --------------------------------------------------------------
        if (eval 'require Term::ReadLine')
        {
            require Term::ReadLine;
            Term::ReadLine->import ();
            $term = new Term::ReadLine;
            $term->ornaments (",,,");
        }
        else
        {
            $term = undef;
        }
        
        # --------------------------------------------------------------
        # | While user has input ...                                   |
        # --------------------------------------------------------------
        $_ = 1;
        while (defined $_)
        {

            # --------------------------------------------------------------
            # | Use ReadLine if we can; otherwise, use mundane input       |
            # --------------------------------------------------------------
            chomp
            (
                $term                           ?
                ($_ = $term->readline ("> "))   :
                ($_ = (print "> " and <STDIN>))
            );

            # --------------------------------------------------------------
            # | Skip blank lines                                           |
            # --------------------------------------------------------------
            next if /^\s*$/;

            # --------------------------------------------------------------
            # | Display current settings                                   |
            # --------------------------------------------------------------
            if (/^\?$/)
            {
                for (qw(module function))
                {
                    printf "%14s  %s\n", ("[" . $_ . "]"), $wine->{$_};
                }
                if (ref ($wine->{args}))
                {
                    printf "%14s  %s\n", ("[args]"), join (', ', @{$wine->{args}});
                }
                else
                {
                    printf "%14s  %s\n", ("[args]"), $wine->{args};
                }
                for (qw(ret_type debug))
                {
                    printf "%14s  %s\n", ("[" . $_ . "]"), $wine->{$_};
                }
                next;
            }
            
            # --------------------------------------------------------------
            # | Eval input                                                 |
            # --------------------------------------------------------------
            ($r = eval) ? print "-> $r\n" : print "$@";

        }  # end while

        print "\x08\x08";

    } # End sub wine_callback



    # ----------------------------------------------------------------------
    # | Allow for script to be called on its own                           |
    # |                                                                    |
    # |                                                                    |
    # |                                                                    |
    # ----------------------------------------------------------------------
    main ();
